#!/usr/bin/perl -w

###############################################################################
## This program COFACTOR has been developed by Ambrish Roy                   ## 
## at Yang Zhang Lab at University of Michigan.                              ## 
## Please address any comments/bug-reports to: ambyroy@umich.edu             ##
###############################################################################


`hostname`=~/(\S+)/;
my $node=$1;
my $datetime=`date`;
my $path=`pwd`;
my $start=time();
print "Hostname: $node\n";
print "Started : $datetime\n";
print "Path    : $path\n";


my $home   = "/nfs/amino-home";
my $librarydir = "/nfs/amino-library";

##################Variables to be set dynamically##############################
#=pod
my $s           = "51";
my $datadir     = "/nfs/amino-home/zhanglabs/IonCom/output/51";   #for seq.txt and init.dat
my $model       = "protein.pdb";
my $modeltype   = "native"; # native or modeled structure
my $tag         = "CF_51_protein";
my $user        = "zhanglabs";
my $predictbions= "YES";
my $run         = "benchmark";
my $coachbin    = "/nfs/amino-home/zhanglabs/IonCom/bin/COACH/bin";
#=cut

=pod
$s           = "CH000001";
$datadir     = "/home/zhanglabs/COACH/output/CH000001";   #for seq.txt and init.dat
$model       = "protein.pdb";
$modeltype   = "native"; # native or modeled structure
$tag         = "CF_CH000001_protein";
$user        = "zhanglabs";
$predictbions= "YES";
$run         = "real";
$coachbin    = "/home/zhanglabs/COACH/bin";
=cut



###############Other parameters chaing only if required #######################
my $modelname   =$model;$modelname=~s/\.pdb//;
my $pfile       ="Bpockets.dat";
my $bsfile      ="Bsites.dat";
my $LIBFILE     ="PDBsearch_$modelname.dat";
my $recorddir   ="$datadir/$modelname/cofactor";
#my $maxtime     ="30"; #I update here
my $maxtime     ="15"; #I update here


if(!-s "$datadir/$model")
{
    die "Error: $datadir/$model was not copied !\n";    
}

unless (-d "$recorddir")
{
    `/bin/mkdir -p $recorddir`;
}
###############Temporary directory for working ################################
my $workdir  ="/tmp/$user/$tag";
`/bin/mkdir -p $workdir`;
chdir "$workdir";
`/bin/rm -rf $workdir/*`;
###########FILES and EXECUTABLES ##############################################
my $COFACTORdir  ="$home/zhanglabs/COFACTOR";
my $bindir       ="$COFACTORdir/bin";
my $datdir       ="$COFACTORdir/bin/dat";
#my $libdir       = "/home/yangji/CASP10/library";
my $libdir       ="$home/yangji/library1/COFACTOR";
#my $libdir      ="/home/ambyroy/DB/COFACTOR/BSITES";

my $nr           ="$librarydir/nr/nr";       
my $blastdir     ="$librarydir/blast";
my $nrpdb        ="$librarydir/PDB/"; 
#my $jsddir       = "$home/yangji/COFACTOR";
###############################################################################
my $email=`cat $datadir/email.txt`;
chomp($email);

my $maxntemplates=500;
if($email =~ /proteinmodelportal/ || $email =~ /yangji/)
{
    $maxntemplates=1000;
}

my $sidcut       = 10.3;
if($run eq 'benchmark')
{
    $sidcut      = 0.30;
}
my $n_temp =75;    ### Number of templates 
my $n_temp2=50;    ### Atleast these many templates to be in processing list after filters (for metal ions)
my $TM_cut =0.5;   ### Proteins of same fold
my $distcut=8;     ### Distance cutoff for ligand clustering
my $tancut =0.7;   ### Tannimoto Coefficient 
my $BScut  =0.8;   ### BS-score cutoff for resonable hits
$predictbions=lc($predictbions);
my $message="";    ### Error logs
###############################################################################



##### Copying files and programs to working directory #########################
`/bin/cp $libdir/BSITES.bsr      $workdir/BSITES.dat`;
`/bin/cp $libdir/BSITES.fpt     $workdir/ligand.fpt`;
`/bin/cp $libdir/NRPDB_BS.map    $workdir/NRPDB_BS.map`;



`/bin/cp $bindir/COFACTOR2        $workdir/Lsearch`;
`/bin/cp $coachbin/Gsearch       $workdir/Gsearch`;
`/bin/cp $bindir/align           $workdir/align`;
`/bin/cp $bindir/TMalign         $workdir/TMalign`;
`/bin/cp $bindir/get_fasta.pl    $workdir/get_fasta.pl`;
#`/bin/cp $bindir/Scwrl4          $workdir/Scwrl4`;
my $scwrl       ="/nfs/amino-home/yzhang/library/yzhang/bin/scwrl4/Scwrl4";


##### Create some other local working directories to decrease IO  #############
unless(-d "$workdir/BSITES")
{
    `/bin/mkdir -p $workdir/BSITES`;
}
my $ALL="$workdir/all";
unless(-d "$ALL")
{  
    `/bin/mkdir -p  $ALL`;
}
my $GBS="$workdir/BSITES/GBS";
unless(-d "$GBS")
{
    `/bin/mkdir -p  $GBS`;  
}
my $COM="$workdir/BSITES/COMPLEX"; 
unless(-d "$COM")
{
    `/bin/mkdir -p  $COM`;  
}
my $REC="$workdir/receptor";
unless(-d "$REC")
{
    `/bin/mkdir -p  $REC`;
}
my $LIG="$workdir/ligand";
unless(-d "$LIG")
{
    `/bin/mkdir -p  $LIG`;
}   
`rm -rf $GBS/*`;
`rm -rf $COM/*`;
`rm -rf $REC/*`;
`rm -rf $LIG/*`;
`rm -rf $ALL/*`;
###############################################################################
#Amino acid tralation table
my %ts=(
	'ALA'=>'A', 'ARG'=>'R', 'ASN'=>'N', 'ASP'=>'D', 'CYS'=>'C', 'GLN'=>'Q', 'GLU'=>'E', 'GLY'=>'G',
	'HIS'=>'H', 'ILE'=>'I', 'LEU'=>'L', 'LYS'=>'K', 'MET'=>'M', 'PHE'=>'F', 'PRO'=>'P', 'SER'=>'S', 
	'THR'=>'T', 'TRP'=>'W', 'TYR'=>'Y', 'VAL'=>'V', 'ASX'=>'N', 'GLX'=>'Q', 'UNK'=>'G',
	'A'=>'ALA', 'R'=>'ARG', 'N'=>'ASN', 'D'=>'ASP', 'C'=>'CYS', 'Q'=>'GLN', 'E'=>'GLU', 'G'=>'GLY',
	'H'=>'HIS', 'I'=>'ILE', 'L'=>'LEU', 'K'=>'LYS', 'M'=>'MET', 'F'=>'PHE', 'P'=>'PRO', 'S'=>'SER', 
	'T'=>'THR', 'W'=>'TRP', 'Y'=>'TYR', 'V'=>'VAL', 'B'=>'ASX', 'Z'=>'GLX', 'X'=>'UNK');
#Definition of biological ions
my %bions=('K'=>1,'MG'=>1,'ZN'=>1,'MN'=>1,'CO'=>1,'FE'=>1,'FE2'=>1,'NI'=>1,'MO'=>1,'CA'=>1,'NA'=>1,'CU1'=>1,'CU2'=>1,'CU'=>1,);		
###############################################################################
###########Read files related to binding site library##########################
my %ligand_print=();
if(!-s "$workdir/ligand.fpt"){$message ="$datdir/ligand.fpt is missing";goto poslast;}
open(FILE,"<$workdir/ligand.fpt");
while(my $line=<FILE>)
{
    chomp($line);
    if($line=~/(\S+)\s+(\S+)/)
    {
	my $ligand_name=$1;
	my $fp=$2;
	$ligand_name=~s/\s+//g;
	$fp=~s/\s+//g;
	$ligand_print{$ligand_name}=$fp;
    }
}
close(FILE);



if(!-s "$workdir/NRPDB_BS.map"){$message ="NRPDB_BS.map is missing";goto poslast;}
my %homologs=();
open(MAP,"<$workdir/NRPDB_BS.map");
while(my $line=<MAP>)
{
    my $center =substr($line,0,15);$center=~s/[\s+|\t]//mg;  
    if($line=~/^\s*$center\s*(\d+)\s+(.*)/)
    {
	my $Nhomologs=$1;
	my @entries=split(/[\s+|\t]/,$2);
	for(my $i=0;$i<=$#entries;$i++)
	{
	    push @{$homologs{$center}},$entries[$i];	  
	}
    }
}
close(MAP);
if(!-s "$workdir/BSITES.dat"){$message ="$datdir/BSITES.dat is missing";goto poslast;}
my %library=();
open(LIB,"<$workdir/BSITES.dat");
while(my $line=<LIB>)
{
    chomp($line);
    my $pdbname=substr($line,0,5);
    $pdbname   =~s/\s+//;
    push @{$library{$pdbname}},$line;    
}
close(LIB);
###############################################################################
############Checking the files#################################################
if(!-d "$datadir")
{
    $message="Error: Dude something is terribly wrong there is no $datadir !!!!\n";
    goto poslast;
}
if(-s "$datadir/$model")
{
    `/bin/cp $datadir/$model         $workdir/$model`;
}
else
{
    $message="Error: $datadir/$model was not copied !\n";
    goto poslast;
}
####  Remove any HETATM and renumber PDB file ##################################
my $n=-1000;my $chain='';my $position=0;my %seq=();
my $last_res_id='';my $last_res_name='';
open(OUT,">temp.pdb");
open(FILE,"<$model");
while(my $line=<FILE>)
{
    if($line=~/^TER/) {goto pdbend;}  # start over if we find a terminus
    if($line=~/^ATOM/)
    {
	
        my $atom1 = substr($line, 12, 2);
        if($atom1 =~/H/g) #ignore hydrogen, which will be used by COFACTOR if not delete it
        {
            next;
        }

	
        my $alt         = substr($line,16,1); #alterate for atom

        if($alt eq " " || $alt eq "A"|| $alt eq "1")
        {
            $position++;
            if ($n == -1000)
            {
                $n = 1;# start with the starting number in the chain
                $last_res_id   = substr($line,22,5);
                $last_res_name = substr($line,17,3);
                $seq{$n}=$ts{$last_res_name};
            }
            my $res_id      = substr($line,22,5);
            my $res_name    = substr($line,17,3);
            if(($res_id ne $last_res_id) || ($res_name ne $last_res_name))
            {
                $n++;
                $last_res_id   = $res_id;
                $last_res_name = $res_name;
                $seq{$n}=$ts{$res_name};
            }
            substr($line,6,5)  = sprintf "%5d",$position;
            substr($line,22,5) = sprintf "%4d ",$n;
            print OUT "$line";
        }
    }
}
 pdbend:;
close(FILE);
print OUT "TER\n";
close(OUT);
my $Lch=$n;
`/bin/mv temp.pdb $model`;
open(FASTA,">$datadir/seq.txt");
print FASTA ">protein\n";
for(my $i=1;$i<=$Lch;$i++)
{
    print FASTA "$seq{$i}";
    if(int($i/60)*60==$i)
    {
        print FASTA "\n";
    }
}
print FASTA "\n";
close(FASTA);

`/bin/cp $datadir/seq.txt $workdir/seq.txt`;
my @seqtxts=();
@seqtxts=`cat $workdir/seq.txt`;
my $sequence1="";
foreach my $seqtxt(@seqtxts)
{
    goto pos6 if($seqtxt=~/\>/);
    $seqtxt=~s/\s//mg;
    $seqtxt=~s/\n//mg;
    $sequence1=$sequence1.$seqtxt;
  pos6:;
}
my $Lch2=length $sequence1;
if($Lch !=$Lch2)
{
    $message="ERROR: There is something wrong in between the query PDB file and sequence file. $Lch != $Lch2";
    goto poslast;
}

open(SEQ,">$workdir/protein.seq");
printf SEQ ">protein\n";
for(my $i=1;$i<=$Lch;$i++)
{
    my $a=substr($sequence1,$i-1,1);
    printf SEQ "$a";
    if($i==int($i/60)*60)
    {
        printf SEQ "\n";
    }
}
close(SEQ);
######### If it is model calculate C-score ####################################
my $MCscore=1;
if($modeltype eq 'model')
{
    my $modelno= $1 if($model=~/\S+(\d+)\.pdb/);
    my %cs   =();
    &getCscore($datadir,\%cs);
    $MCscore=($cs{$modelno} + 5)/7;   
    $MCscore= 0.1 if ($MCscore < 0.1);
}
### Get Shannon-divergence score based on MSA of PSI-BLAST search#############
if(!-s "$datadir/JSD.dat")
{
    &runJSD($s,$datadir,$user,$blastdir,$nr,$bindir);
    `/bin/cp $datadir/JSD.dat    ./JSD.dat`;
    `/bin/cp $datadir/SOP.dat   ./SOPM.dat`;
}
else
{
    `/bin/cp $datadir/JSD.dat    ./JSD.dat`;
    `/bin/cp $datadir/SOP.dat   ./SOPM.dat`;
}
my %profile=(); my $profilecheck=0;
open(PROFILE,"<$workdir/JSD.dat") || print "ERROR: JSD is missing\n";
while(my $line=<PROFILE>)
{
    if($line=~/(\S+)\s+(\S+)/)
    {
        $profile{$1}=$2;
	if($2 > 0.5){$profilecheck++;}
    }
}
close(PROFILE);
##### Incase if first profile fails 
if($profilecheck < 1)
{
    open(PROFILE,"<$workdir/SOPM.dat") || print "ERROR: SOPM.dat is missing\n";
    while(my $line=<PROFILE>)
    {
	if($line=~/(\S+)\s+(\S+)/)
	{
	    $profile{$1}=$2;	   
	}
    }
    close(PROFILE);
}
#### Run TM-align on NR PDB library (fastero###################################
#.1 If the result is already present copy search result
if((-e "$datadir/$LIBFILE") && (-s "$datadir/$LIBFILE" > 50))
{
     `/bin/cp $datadir/$LIBFILE .`;
}
#. Else run TM-align search
else
{
     system("./Gsearch $model $nrpdb > $LIBFILE");
     `/bin/cp $LIBFILE $datadir/.`;
}
##############Read TM-align results ###########################################
my $iter = 0;
 postart:;
my $i=0; my %targetname=(); my %tmscore1=(); my %tmscore2=(); my %FH=();
$iter++;
if(-s "$workdir/$LIBFILE" <100){$message ="$workdir/$LIBFILE was not generated correctly. If it exists in $datadir, delete it and run again";goto poslast;}
open(RESULT,"<$workdir/$LIBFILE") || die "Global search results are absent\n";
while(my $line=<RESULT>)
{
    chomp($line);
    if($line=~/Template=\s*(\S+)\s*,\sQuery=\s*\S+\s*,\sLch_t=\s*\d+\,\sLch_q=\s*\d+\,\sTM-score_q=\s*(\S+)\,\sTM-score_t=\s*(\S+)\,\sALN=\s*\d+\,\sSID=\s*\S+/)
    {
	$targetname{$i}=$1;
	$tmscore1{$i}=$2;
	$tmscore2{$i}=$3;
	$i++;
    }
  posskip:;
}
close(RESULT);

#######Sort  based on TM-score and get hits from Binding site library##########
my @keys=(); 
@keys = reverse sort{$tmscore1{$a} <=> $tmscore1{$b}} keys %tmscore1;
 post:;
my %seen_template=();my %TMscore=();my %RMSDa=(); my %IDEN=(); my %Cov=(); my %seen_temp=();
my @selected1=();my $count1 =0;my $count2=0;
my $seedtm=0;
for(my $j=0;$j<$#keys;$j++)
{
    my $seed1         =$targetname{$keys[$j]};  
    if(exists $homologs{$seed1})
    {        
	$seedtm        =$tmscore1{$keys[$j]};	
	$seed1            =~s/\s+//g;
	my @templates=@{$homologs{$seed1}};
	print "$seed1\t$seedtm\t @templates\n";
	foreach my $template(@templates)
	{
	    my $ID  = 0.0;
	    my @ligN=@{$library{$template}};
	    foreach my $line(@ligN)
            {
	       my $pdbid = substr($template,0,4);
               my $lig   = substr($line,7,3);$lig =~s/\s+//;
#	       if(exists $seen_temp{$pdbid,$lig}){goto next_template;}
	    }
	    my $pdb="$libdir/receptor/$template\.pdb";
	    `/bin/cp $pdb $ALL/$template\.pdb`;

	    if($sidcut<1)
	    {
		my $rst=`$workdir/align protein.seq $ALL/$template\.pdb 2`;
		if($rst=~/Identical length:\s*(\d+)/)
		{
		    $ID=$1/$Lch;
		}
		print "$template\t$ID\n";
		if($ID >= $sidcut){next;}
	    }	    
	    if($template ne '')
	    {
		`/bin/cp $ALL/$template\.pdb $REC/$template\.pdb`;	
		my $rst2='';
		$rst2=`./TMalign $ALL/$template\.pdb $model`;
		if($rst2=~/TM-score\s*=(\S+),/)
		{
		    $TMscore{$template} = $1;
		    print "TMscore=$TMscore{$template}\n\n";
		    if( $TMscore{$template} <$TM_cut)
		    {
			next;
		    }
		}
		if($rst2=~/Aligned length\s*=\s+(\d+),/)
		{
		    $Cov{$template} = $1/$Lch;		 
		}
		if($rst2=~/RMSD=\s*(\S+),/)
		{
		    $RMSDa{$template} = $1;
		}
		if($rst2=~/ID=(\S+)/)
		{
		    $IDEN{$template}=$1;
		}
		$count1++;
		push(@selected1,$template);
		$seen_template{$template}=1;
		foreach my $line(@ligN)
            	{
		    my $pdbid = substr($template,0,4);
		    my $lig   = substr($line,7,3);$lig =~s/\s+//;
		    $seen_temp{$pdbid,$lig}=1;
            	}
		$count2 += ($#ligN + 1);
		`/bin/cp $libdir/ligand/$template\_* $LIG/.`;
	    }
	    next_template:;
	}
	print "count2=$count2\n";
	if($seedtm < $TM_cut) ##No more good templates
	{
	    print "$seedtm<$TM_cut and $count2 \n";
	    goto loopout1;
	}
	
	if($count2 > ($n_temp+50) ) ##the number of good templates are enough no matter if there are other good templates
	{
	    print "$count2>$n_temp+50\n";
	    if($seedtm < $TM_cut+0.1 || $count2>$maxntemplates)
	    {
		goto loopout1; #I update here
	    }
	}
    }
}
 loopout1:;

 
#exit;
######Make Process list########################################################
my $count3=0;
open(OUT1,">$workdir/process.lst");
foreach my $template(@selected1)
{
    if(exists $library{$template})
    {
       my @ligN=@{$library{$template}};
       foreach my $line(@ligN)
       {
	   my $lig = substr($line,7,3);$lig =~s/\s+//;	  
	   if((exists $bions{$lig})&& ($predictbions eq 'no')){next;} ##### Remove for real runs
	   print OUT1 "$line\n";
	   $count3++;
       }
    }
}
loopout2:;
close(OUT1);
if($count3 < $n_temp2 && $seedtm < $TM_cut) #reduce cutoff and restart with new cutoff TM_cut
{    
    if($predictbions eq 'no')
    {
	$n_temp= $n_temp + 10;
    }
    $TM_cut -= 0.1 if($TM_cut>0.1);  #I update here
    print "\n\n*************count3=$count3 < $n_temp2\n reselect with new TM_cut=$TM_cut and  n_temp = $n_temp*************\n\n";
    goto postart;
}


######### Run Local search on selected templates ##############################
my $Pcount = 0;my $TNBS=0;
open(RECORD,">$workdir/record.dat");
open(RESULT,">$COM/unsorted.lst");
open(PROCESS,"<$workdir/process.lst");
while(my $line=<PROCESS>)
{
    chomp($line);
    my $template = ''; my $bsno     = '';my  $no_bsres = '';my $bindingr = '';
    if($line=~/(\S+)\s+\:\S+\:(\S+)\:(\d+)\:(\S+)/)
    {
        $template = $1;
        $bsno     = $2;
        $no_bsres = sprintf("%d",$3);
        $bindingr = $4;
    }
    elsif($line=~/(\S+)\s+\: \S+\:(\S+)\:(\d+)\:(\S+)/)
    {
        $template = $1;
        $bsno     = $2;
        $no_bsres = sprintf("%d",$3);
        $bindingr = $4;
    }
    elsif($line=~/(\S+)\s+\:\S+ \:(\S+)\:(\d+)\:(\S+)/)
    {
        $template = $1;
        $bsno     = $2;
        $no_bsres = sprintf("%d",$3);
        $bindingr = $4;
    }
    elsif($line=~/(\S+)\s+\:  \S+\:(\S+)\:(\d+)\:(\S+)/)
    {
        $template = $1;
        $bsno     = $2;
        $no_bsres = sprintf("%d",$3);
        $bindingr = $4;
    }    
    else{print "$line dosent matches\n";goto nexttemplate;}
    $Pcount++;
    my @TP_bsres=split(/,/,$bindingr);
    $TNBS += $#TP_bsres;
   
    my $structure1="$datadir/$model";
    my $structure2="$workdir/receptor/$template.pdb"; 
    my %PBS=();	
    $PBS=&getBinding($structure1, $structure2,$workdir,$datadir,$Lch,$bindingr,\%profile,$blastdir,$MCscore);
    my $profile_file= "$GBS/$template\_$bsno.dat";
    my $pairs=0;
    open(GBSR,">$profile_file");
    for(my $i=1;$i<=$Lch;$i++)
    {
	if($$PBS{0,$i} > 0){$pairs++;}
	printf GBSR "%-4d  %5.3f\n",$i,$$PBS{0,$i};
    }
    close(GBSR);
  
    open(PROC,">$workdir/list");
    print PROC "$line\n";
    close(PROC);
    my @rst=(); my @residues=();my %clash=();
    my $ligand  ='';my $bscore=0;my $rmsd=0;my $sid=0;my $coverage=0;my $disterr=0;my $idali1=0;my $idali2=0;
    @rst=`./Lsearch $profile_file $model $workdir -c `;
    foreach my $rline(@rst)
    {
	if($rline=~/Template=\s*(\S+)\s+(\S+)\s+(\S+),\s+BS=(\S+),\s+LTM=(\S+),\s+SID=(\S+),\s+COV=(\S+),\s+ERR=\s*(\S+),\s+IDALI1=(\S+),\s+IDALI2=(\S+)/)
	{
	    $template=$1;
	    my $pdbid   =substr($template,0,4);$pdbid   =lc($pdbid);
	    my $pdbchain=substr($template,4,1);$pdbchain=uc($pdbchain);
	    $ligand  =$2;
	    $bsno    =$3;
	    $bscore  =$4;
	    $rmsd    =$5;
	    $sid     =$6;
	    $coverage=$7;
	    $disterr =$8;
	    $idali1  =$9;
	    $idali2  =$10;
	}
	elsif($rline=~/\s+\d+\s+\S+\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\*/)
	{
	    my $r    =$1;
	    push(@residues,$r);
	}
	elsif($rline=~/\s*\CLASH=\s*(\S+)/)
	{
	    $clash{$template,$bsno}=$1;
	}	
	else
	{
	    next;
	}	
    }
    if($#residues>=0)
    {
	printf RESULT "%-5s %5.4f %5.4f %5.4f %5.4f %3s %4s %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f :", $template, $TMscore{$template}, $RMSDa{$template}, $IDEN{$template}, $Cov{$template}, $ligand, $bsno, $bscore, $rmsd, $sid, $coverage, $disterr, $idali1,$idali2;  
	printf RESULT join(',',@residues);
	printf RESULT "\n";
	`$scwrl -i $model -o temp.pdb -f BSITE_$template\_$bsno.pdb -0 `;	   
        if(-s "temp.pdb")
        {	    
	    `/bin/cat temp.pdb BSITE_$template\_$bsno.pdb > $COM/$modelname\_$template\_$bsno.pdb`;
	    `rm temp.pdb`;
	}
	else
        {
	    `/bin/cat $model BSITE_$template\_$bsno.pdb > $COM/$modelname\_$template\_$bsno.pdb`;
        }
    }
    print RECORD "@rst\n";
    my $runtime = runhour($start);
    if(($maxtime - $runtime) <= 0.2)    
    {
	print "maxtime-runtime=$maxtime - $runtime <= 0.2, jump out from local search\n";
	goto finishs;
    }
  nexttemplate:;
}
finishs:;
close(PROCESS);
close(RECORD);
close(RESULT);

`/bin/cp -r process.lst $recorddir/.`;
#`/bin/cp -r $COM        $recorddir/.`;
#`/bin/cp -r $GBS        $recorddir/.`;
`/bin/cp -r record.dat  $recorddir/.`;



######### Cluster the predicted ligands #######################################
my %xs=(); my %ys=(); my %zs=();
%TMscore =(); %RMSDa  =(); %IDEN =(); %Cov   =(); my %PC    =();
$i=0;
open(PDB,"$workdir/$model")|| goto nextpro;
while(my $line2=<PDB>)
{
    my $record     = substr($line2,0,6);
    if($record eq "ATOM  ")
    {
	my $atom_name = substr($line2,12,4);
	my $res_no    = substr($line2,22,4);$res_no    =~s/\s+//;
	if($atom_name eq ' CA ')
	{
	    $xs{$res_no} =substr($line2,30,8);
	    $ys{$res_no} =substr($line2,38,8);
	    $zs{$res_no} =substr($line2,46,8);
	}
    }
}
close(PDB);
if(!-s "$COM/unsorted.lst"){print "Error: RESULTS from the local search is missing. Remove everything and rerun.\n";goto poslast;}
reclust:;
my %hits    =(); my %ligand_name=(); my %ligands=();
my %BS_score=(); my %LTM     =(); my %BS_ID=(); my %BS_cov=(); my %BS_err=(); my %BS_ID1     =(); my %BS_ID2 =();
my %PBS     =(); $i=0;my $Ncluster=0;

open(SEL, ">selected_templates.lst");
my %Rank_scores=();
open(RESULT,"<$COM/unsorted.lst");
while(my $line2=<RESULT>)
{
    if($line2=~/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\:(\S+)/)
    {
        $hits{$i}        =$1;
        $TMscore{$i}     =$2;
        $RMSDa{$i}       =$3;
        $IDEN{$i}        =$4;
        $Cov{$i}         =$5;
        $ligand_name{$i} =$6;
        $ligands{$i}     =$7;
        $BS_score{$i}    =$8;
        $LTM{$i}         =$9;
        $BS_ID{$i}       =$10;
        $BS_cov{$i}      =$11;
        $BS_err{$i}      =$12;
        $BS_ID1{$i}      =$13;
        $BS_ID2{$i}      =$14;
        $PBS{$i}         =$15;

	my @superlig=();
        my $xcom=0;my $ycom=0;my $zcom=0; my $natoms=0;
        open(FILE,"<$COM/$modelname\_$hits{$i}\_$ligands{$i}\.pdb") || print "$COM/$modelname\_$hits{$i}\_$ligands{$i}\.pdb was missing\n" ;
        while(my $l=<FILE>)
        {
            if($l=~/^HETATM/)
            {
                $natoms++;
                $xcom +=substr($l,30,8);
                $ycom +=substr($l,38,8);
                $zcom +=substr($l,46,8);

		chomp($l);
		push(@superlig, $l);
            }
        }
        close(FILE);
	next if($natoms==0);
	$PC{$i,1} = $xcom/$natoms;
	$PC{$i,2} = $ycom/$natoms;
	$PC{$i,3} = $zcom/$natoms;

	#Filters
	if((exists $bions{$ligand_name{$i}})&& ($BS_ID2{$i} < 0.60))
	{
	    print "$hits{$i}\t $ligand_name{$i} is bion with $BS_ID2{$i} < 0.6, ignore this template\n";
	    next;
	}
        if($BS_ID2{$i}   < 0.1)   
	{
	    print "$hits{$i}\t $ligand_name{$i}  BS_ID2 $BS_ID2{$i}< 0.6, ignore this template\n";
	    next;
	}
        if($BS_score{$i} < $BScut)
	{
	    print "$hits{$i}\t $ligand_name{$i}  BS_score $BS_score{$i} < $BScut, ignore this template\n";
	    next;
	}
        $Rank_scores{$i} =  $TMscore{$i} + 2.5*$IDEN{$i} + (0.25* $BS_score{$i});
	print "$hits{$i}\t $ligand_name{$i}  Rank_scores=$Rank_scores{$i}, keep this template\n";

	open(LIG, ">lig_$hits{$i}\_$ligands{$i}\_$ligand_name{$i}.pdb");
	print SEL $line2;
	print LIG join("\n", @superlig);	        
	print LIG "\nTER COFACTOR $hits{$i}\_$ligands{$i}\_$ligand_name{$i}\n";
	$i++;
    }
}
close(RESULT);

close(SEL);
print LIG "END\n";
close(LIG);
`cp selected_templates.lst $recorddir/`;
`cp lig_*.pdb $recorddir/`;

my $NBST=$i;
print "No of BS: $NBST\n";

if(($NBST < 5) && ($BScut > 0.5)){$BScut =  $BScut - 0.1;goto reclust;}
if(($NBST < 1) && ($BScut > 0.1)){$BScut =  0.1;goto reclust;}
if($NBST < 1){goto nobs;}
############# Perform Clustering based on Euclidean distance of Binding Site Center and Tannimoto Coeff of ligands###
my %ED=(); my %TN=(); my %Lname=();
for(my $i=0;$i<$NBST;$i++)
{
    $ED{$i,$i}=0;
    $TN{$i,$i}=1;
    my $ligname1=$hits{$i}."_".$ligands{$i};
    print "$ligname1\n";
    $Lname{$i}=$ligand_name{$i};
    for(my $j=$i+1;$j<$NBST;$j++)
    {
        if($i == $j){next;}
        $ED{$j,$j}=0;
        $TN{$j,$j}=1;
        my $dist =sqrt(($PC{$i,1}-$PC{$j,1})**2+
                       ($PC{$i,2}-$PC{$j,2})**2+
                       ($PC{$i,3}-$PC{$j,3})**2);
        $ED{$i,$j}=sprintf("%6.3f",$dist);
        $ED{$j,$i}=sprintf("%6.3f",$dist);
        my $ligname2=$hits{$j}."_".$ligands{$j};
        my $tani = 0.0;
        $tani =&tannimoto($ligand_print{$ligname1},$ligand_print{$ligname2});
        if((exists $bions{$ligand_name{$i}}) && (exists $bions{$ligand_name{$j}}))
        {
            $tani = 1;
        }
        elsif(($ligand_name{$i} eq 'NUC') && ($ligand_name{$j} eq 'NUC')) #I update here
        {
            $tani = 1;
        }

        $TN{$i,$j}=$tani;
        $TN{$j,$i}=$tani;
    }
}

my $prediction_no=0; my %clusters=();my %pockets=(); my %Cscore=(); my %BSRP=();

print "Doing Clustering\n";

%clusters=&cluster($recorddir,$Lch,\$Ncluster,$distcut,$tancut,$NBST,\%ED,\%TN,\%Rank_scores,\%BS_score,\%BS_ID1,\%PBS,\%Lname,\%pockets,\%PC,\%Cscore,\%BSRP);
 nobs:;
open(RESULT3,">$recorddir/Bsites_prob.dat");
open(RESULT2,">$recorddir/$pfile");
open(RESULT1,">$recorddir/$bsfile");
if($Ncluster >=1)
{
    foreach my $rank (sort {$Cscore{$b} <=> $Cscore{$a} } keys %Cscore)
    {
        my $index = $clusters{$rank};
        $prediction_no++;
        printf RESULT2 "POCKET %-2d %5.3f %8.3f %8.3f %8.3f\n",$prediction_no,$Cscore{$rank},$pockets{$rank,1},$pockets{$rank,2},$pockets{$rank,3};
	printf RESULT1 "%-4d %5.3f %-5s %5.4f %5.4f %5.4f %5.4f %3s %4s %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f:$PBS{$index}\n",$prediction_no,$Cscore{$rank},$hits{$index}, $TMscore{$index}, $RMSDa{$index}, $IDEN{$index}, $Cov{$index}, $ligand_name{$index},$ligands{$index} ,$BS_score{$index}, $LTM{$index},$BS_ID{$index},$BS_cov{$index},$BS_err{$index},$BS_ID1{$index},$BS_ID2{$index};
	`/bin/cp $COM/$modelname\_$hits{$index}\_$ligands{$index}.pdb $recorddir/complex$prediction_no\.pdb`;	
	for (my $l=1;$l<=$Lch;$l++)
	{
	    #print RESULT3 "$BSRP{$prediction_no,$l} "; #I update here;
	    print RESULT3 "$BSRP{$rank, $l} ";
	}
	print RESULT3 "\n";
    }
}  
else
{    
    printf RESULT2 "No Binding pocket predicted\n";
    printf RESULT2 "No Binding pocket predicted\n";
    for (my $iter=1;$iter<=5;$iter++){
       for (my $l=1;$l<=$Lch;$l++){
          print RESULT3 "0.00 ";
       }	
       print RESULT3 "\n";
    }
}
close(RESULT1);
close(RESULT2);
close(RESULT3);
################# Endding the procedure #######################################
 poslast:;
print "$message\n";
my $time=`date`;
printf "ending time: $time";
`sync`;
`sync`;
sleep(1);
`rm -fr $workdir`;
exit();


###############################################################################
#########*********SUBROUTINES***********########################
###Cluster predicted ligands from templates
sub cluster
{
    my($dir,$Lch,$Ncluster,$ED_cut,$TAN_cut,$NBS,$ED_ref,$TN_ref,$RS_ref,$LS_ref,$ID_ref,$PB_ref,$name_ref,$pocket_ref,$PC_ref,$CS_ref,$BSRP_ref)=@_;
    my %EDh   = %$ED_ref;
    my %TNh   = %$TN_ref;
    my %RSh   = %$RS_ref;
    my %LSh   = %$LS_ref;
    my %PBh   = %$PB_ref;
    my %bions=('MG'=>1,'ZN'=>1,'MN'=>1,'CO'=>1,'FE'=>1,'FE2'=>1,'NI'=>1,'MO'=>1,'CA'=>1,'NA'=>1,'CU1'=>1,'CU2'=>1,'CU'=>1);

    my %clusters=(); my %bool   =();
    my %Dmatrix =(); my %Cmatrix=();
    my %NEB=();

    # Parameters for very good match 
    my $good_local = 1.3; my $good_id = 0.60;

    # Store all the values
    for(my $id1=0;$id1<$NBS;$id1++)
    {
        $bool{$id1}=1;
        for(my $id2=0;$id2<$NBS;$id2++)
        {
            my $dist= $EDh{$id1,$id2};
            my $tan = $TNh{$id1,$id2};
            $Dmatrix{$id1,$id2}=$dist;
            $Cmatrix{$id1,$id2}=$tan;
        }
    }
    my $i =0;
    do
    {
        my %ND=();my %NS=();my %goodM=(); my %DSsim=(); my %CSsim=();
        for(my $id1=0;$id1<$NBS;$id1++)
        {
            if($bool{$id1}==1)
            {
                $ND{$id1}=0;$NS{$id1}=0; push @{$goodM{$id1}},$id1;
                for(my $id2=0;$id2<$NBS;$id2++)
                {
                    if(($bool{$id2} == 0) || ($id1 == $id2)){next;}
                    my $dist= $EDh{$id1,$id2};
                    my $tan = $TNh{$id1,$id2};
                    if($dist <= $ED_cut)                    
		    {
                        $ND{$id1}++;
                        push @{$goodM{$id1}},$id2;
                    }
                    if($tan >= $TAN_cut)
                    {
                        $NS{$id1}++;
                    }
                }
                if(exists $NEB{$id1}){next;}
                else{$NEB{$id1}=$ND{$id1};}
            }
            else
            {
                next;
            }
        }

        my @ranking1 = reverse sort{$ND{$a} <=> $ND{$b}} keys %ND;
        my $id1 = $ranking1[0]; my $id1_score=$RSh{$id1}; my $id1_iden=$$ID_ref{$id1};my $id1_local=$LSh{$id1};
        my @col=@{$goodM{$id1}};
        my $arrsize= $#col + 1;     
        foreach my $C1 (@col)
        {
            $DSsim{$C1}=1;
            foreach my $C2 (@col)
            {
                if($C1 eq $C2){$DSsim{$C1}+=0;$CSsim{$C1} += 0; next;}
                $DSsim{$C1} += $EDh{$C1,$C2};
                $CSsim{$C1} += $TNh{$C1,$C2};
            }
            $DSsim{$C1} = $DSsim{$C1}/$arrsize;
            $CSsim{$C1} = $CSsim{$C1}/$arrsize;
        }
        my $mean=&Mean(\%DSsim,\@col,$arrsize);
        my $SD  =&SD(\%DSsim,\@col,\$mean,$arrsize);
        my %SS=();my $density = ($#col+1)/$NBS;
        foreach my $C1(@col)
	{
            my $y        = ($RSh{$C1} + 2/(1 + $DSsim{$C1}));
            if($#col <10){$y     = $RSh{$C1};}
            my $density  = ($NEB{$C1}/$NBS);
            if(($density <0.35) && ($LSh{$C1} > $good_local) && ($$ID_ref{$C1} > $good_id)){$density=0.35;}
            my $x        = - $density *($y- 0.8);
	    $density     = 0.1 if($density < 0.1);
            my $CscoreLB   = (2/(1+exp($x)) + 0.01)-1;
            $SS{$C1}     = $CscoreLB;
        }
        my @ranking2 = reverse sort{$SS{$a} <=> $SS{$b}} keys %SS;
        my $selected = $ranking2[0];
	my %BSP=();
        for(my $l=1;$l<=$Lch;$l++){$BSP{$l}=0.00;}
        my $xc = $$PC_ref{$selected,1}; my $yc = $$PC_ref{$selected,2}; my $zc = $$PC_ref{$selected,3}; my $in_cluster=1;
        my $id = $selected;
        my $n  = $$name_ref{$id};
        foreach my $id2 (@col)
        {
            if($id eq $id2){next;}
            my $bs_sim1=&bssim($$PB_ref{$id},$$PB_ref{$id2});
            my $bs_sim2=&bssim($$PB_ref{$id2},$$PB_ref{$id});
            if(($EDh{$id,$id2} <= $ED_cut) && ($TNh{$id,$id2} >= $TAN_cut) && (($bs_sim1 >= 0.50) || ($bs_sim2 >= 0.50)))
            {
                my $Lname1 = $$name_ref{$id}; $Lname1 =~s/\s+//;
                my $Lname2 = $$name_ref{$id2};$Lname2 =~s/\s+//;
                if((exists $bions{$Lname1})&&(exists $bions{$Lname2}))
                {
                    if($EDh{$id,$id2} <= 4.0)
                    {
                        $bool{$id2}=0;$i++;
                        $xc +=  $$PC_ref{$id2,1};
                        $yc +=  $$PC_ref{$id2,2};
                        $zc +=  $$PC_ref{$id2,3};
                        $in_cluster++;
			my @res=split(/,/,$PBh{$id2});
			foreach my $r(@res){$BSP{$r}++;}
                    }
                }
                elsif(exists $bions{$Lname2})
                {
                    next;
                }
                else
                {
                    $bool{$id2}=0;$i++;
                    $xc += $$PC_ref{$id2,1};
                    $yc += $$PC_ref{$id2,2};
                    $zc += $$PC_ref{$id2,3};
                    $in_cluster++;
		    my @res=split(/,/,$PBh{$id2});
		    foreach my $r(@res){$BSP{$r}++;}
                }
            }
            elsif(($EDh{$id,$id2}  <= $ED_cut) && ($bs_sim1 >= 0.60) && ($bs_sim2 >= 0.60))
            {
                $bool{$id2}=0;$i++;
                $xc += $$PC_ref{$id2,1};
                $yc += $$PC_ref{$id2,2};
                $zc += $$PC_ref{$id2,3};
                $in_cluster++;
		my @res=split(/,/,$PBh{$id2});
                foreach my $r(@res){$BSP{$r}++;}
            }
        }
        $$Ncluster++;
        $$pocket_ref{$$Ncluster,1}=$xc/$in_cluster;
        $$pocket_ref{$$Ncluster,2}=$yc/$in_cluster;
        $$pocket_ref{$$Ncluster,3}=$zc/$in_cluster;

	my @res=split(/,/,$PBh{$selected});
	foreach my $r(@res){$BSP{$r}++;}

	for (my $l=1;$l<= $Lch; $l++)
	{	    
	    $$BSRP_ref{$$Ncluster,$l} = sprintf("%4.2f",($BSP{$l}/$in_cluster));	    
	}
        $clusters{$$Ncluster}=$selected;
        $$CS_ref{$$Ncluster}=$SS{$selected};
        $bool{$selected}=0;
        $i++;
    }while($i<$NBS);
    return(%clusters);
}
 
#Get potential binding site locations in query structure
sub getBinding
{
  my($pdbfile1, $pdbfile2,$dir,$datadir,$Lch,$bindingr,$seqcons,$blast,$CS) = @_;
  my %profile=%$seqcons; my @rst=();
  my %PBS=();my %QBS=();
  #####################Template binding residues
  my %TBS=(); my $NTBR=0;
  my @Tres=split(/\,/,$bindingr);
  foreach my $res(@Tres)
  {
      $res=sprintf("%d",$res);
      $TBS{$res}=1;     
  }
  $NTBR=@Tres;

  ##### Get Geometric Center of Template Binding site and Radius of Template binding site
  my $GCX=0.0; my $GCY=0.0; my $GCZ=0.0;
  my %X  = (); my %Y  = (); my %Z  = ();
  my $R=0;my $nres=0;
  
  open(TEMPL,"<$pdbfile2") || die "Could not find $pdbfile2\n";
  while(my $line=<TEMPL>)
  {
      if($line=~/^ATOM/){
	  if(substr($line,12,4)=~/CA/){
	      $nres++;
	      if(exists $TBS{$nres}){
		  $GCX +=substr($line,30,8);
		  $GCY +=substr($line,38,8);
		  $GCZ +=substr($line,46,8);		  
	      }
	      $X{$nres}=substr($line,30,8);
	      $Y{$nres}=substr($line,38,8);
	      $Z{$nres}=substr($line,46,8);
	  }
      }
  }
  close(TEMPL);
  $GCX=$GCX/$NTBR;
  $GCY=$GCY/$NTBR;
  $GCZ=$GCZ/$NTBR;
  my $pcut=0;
  my $nbres1=0;
  for(my $i=1;$i<=$nres;$i++)
  {
      if(exists $TBS{$i})
      {
	  $nbres1++;
	  my $dis=sqrt(($X{$i}-$GCX)**2+
		       ($Y{$i}-$GCY)**2+
		       ($Z{$i}-$GCZ)**2);
	  if($dis > $R)
	  {
	      $R=sprintf("%6.3f",$dis);
	  }
      }
  }
  ##########Superpose Query model and Template & get Rotation and Translation matrix
  my @rot=();my @tra=();
  @rst=(); my $TM =0;
  @rst=`$dir/TMalign $pdbfile1 $pdbfile2`;
  my @aln1=();my @aln2=();
  for(my $i=0;$i<$#rst;$i++)
  {
     if($rst[$i]=~/\s+1\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
     {
      $tra[0]=$1;
      $rot[0][0]=$2;
      $rot[0][1]=$3;
      $rot[0][2]=$4;
     }
     if($rst[$i]=~/\s+2\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/){
      $tra[1]=$1;
      $rot[1][0]=$2;
      $rot[1][1]=$3;
      $rot[1][2]=$4;
     }
     if($rst[$i]=~/\s+3\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/){
      $tra[2]=$1;
      $rot[2][0]=$2;
      $rot[2][1]=$3;
      $rot[2][2]=$4;
     }
     if($rst[$i]=~/TM-score\s*=(\S+),/)
     {
        $TM = $1;
     }
     if($rst[$i]=~/^\(/)
     {
        @aln1=split(//,$rst[$i+1]);
        @aln2=split(//,$rst[$i+3]);
     }
  }
  if($TM >= 0.45)
  {
      $pcut =0.25;
  }
  my $Qindex=0;my $Tindex=0;
  for(my $pos=0; $pos<$#aln1; $pos++)
  {
      if ($aln1[$pos] ne "-" && $aln2[$pos] ne "-")
      {
          if(exists $TBS{$Tindex}){$QBS{$Qindex}=1;}
      }
      if ($aln1[$pos] ne "-")
      {
          $Qindex++;
      }
      if ($aln2[$pos] ne "-")
      {
          $Tindex++;
      }
  }
  ##########Superpose Query model and get probable binding site region in the model######################
  for (my $i=1;$i<=$Lch;$i++)
  {
      $PBS{0,$i}=-1;
      $PBS{1,$i}=-1;
      if($profile{$i} >=$pcut)
      {
	 $PBS{0,$i}=$profile{$i}; 
      }
      if(($CS < 0.4) && ($profile{$i} > 0.0))
      {
	  $PBS{0,$i}=0.31;
      }
  }
  my $qres = -1000;my $start = 1;my $last_res_id=''; 
  open(QUE,"<$pdbfile1");
  while(my $line=<QUE>)
  {
      if($line=~/^ATOM/)
      {
	  if ($qres == -1000)
	  { # start with the starting number in the chain	     
	      $qres = $start;
	      $last_res_id = substr($line,22,5);
	  }
	  my $res_id = substr($line,22,5);
	  if ($res_id ne $last_res_id)
	  {
	      $qres++;
	      $last_res_id = $res_id;
	  }
	  my $xc=substr($line,30,8);
	  my $yc=substr($line,38,8);
	  my $zc=substr($line,46,8);
	   
	  my $nx=$rot[0][0]*$xc+$rot[0][1]*$yc+$rot[0][2]*$zc+$tra[0];
	  my $ny=$rot[1][0]*$xc+$rot[1][1]*$yc+$rot[1][2]*$zc+$tra[1];
	  my $nz=$rot[2][0]*$xc+$rot[2][1]*$yc+$rot[2][2]*$zc+$tra[2];
	  my $dis=sqrt(($nx-$GCX)**2+
		       ($ny-$GCY)**2+
		       ($nz-$GCZ)**2);
	  my $cut = 2;        
          my $nbres2= 0;
	  if($dis <= $R+$cut)
	  {	 
	      if($profile{$qres} >=$pcut)
	      {
		  $PBS{0,$qres}=$profile{$qres};
		  $nbres2++;
	      }
	      elsif(exists $QBS{$qres})
	      {
		  $PBS{0,$qres}=1;
	      }
	  }
      }
  }
  close(QUE);
  system("./get_fasta.pl $pdbfile2 template.seq");
  my @blast_out=`$blast/bin/bl2seq -i protein.seq -j template.seq  -p blastp > blast_align.out`;
  open(BLA,"<blast_align.out");
  while(my $bline=<BLA>)
  {  
      if($bline=~/Expect =\s*(\S+)/)
      {
	  <BLA>;
	  <BLA>;
	  while($bline=<BLA>)
	  {
	      if($bline=~/Query\:\s*(\d+)\s+(\S+)\s+(\S+)/)
	      {
		  my $i1=$1;
		  my $seq1=$2;
		  <BLA>;
		  <BLA>=~/Sbjct\:\s*(\S+)\s+(\S+)\s+(\S+)/;
		  my $seq2=$2;
		  my $j1=$1;
		  <BLA>;
		  ###
		  my $L=length $seq1;
		  my $m1=$i1-1;
		  my $n1=$j1-1;
		  for(my $i=1;$i<=$L;$i++)
		  {
		      my $q1=substr($seq1,$i-1,1);
		      my $q2=substr($seq2,$i-1,1);
		      $m1++ if($q1 ne '-');
		      $n1++ if($q2 ne '-');
		      if($q1 ne '-' && $q2 ne '-')
		      {			 
			  if(exists $TBS{$n1})
			  {
			      $PBS{0,$m1}=1;
			  }
		      }
		  }		  
	      }
	      else
	      {
		  goto pos11;
	      }	      
	  }
	pos11:;
      }
  }
  close(BLA);
  return \%PBS;
}

### Get Shannon-divergence score based on MSA of PSI-BLAST search
sub runJSD
{
    (my $protname, my $dir1, my $user, my $blast, my $NR, my $bin)=@_;
    my $mod=`cat $bin/JSDmod`; #I change it here
    my $tag="JSD_$protname"; # unique name
    my $jobname="$tag";
    $mod=~s/\!S\!/$protname/mg;
    $mod=~s/\!DATADIR\!/$dir1/mg;
    $mod=~s/\!USER\!/$user/mg;
    $mod=~s/\!BLASTDIR\!/$blast/mg;
    $mod=~s/\!NR\!/$NR/mg;
    $mod=~s/\!BINDIR\!/$bin/mg;
    open(FH1,">$jobname");
    print FH1 "$mod\n";
    close(FH1);
    `chmod a+x $jobname`;
    `/bin/cp $jobname $dir1/.`;
    `./$jobname`;
    return;
}

### Identity of binding sites
sub bssim
{
    my($f1,$f2)=@_;
    my @f1bin=split(/\,/,$f1);
    my @f2bin=split(/\,/,$f2);
    my $NAB=0; my $SIM=0; my %seen_res1=();
    my $d = $#f1bin + 1;
    for(my $i=0;$i<=$#f1bin;$i++)
    {
        $seen_res1{$f1bin[$i]}=1;
    }
    for(my $i=0;$i<=$#f2bin;$i++)
    {
        $NAB++ if(exists $seen_res1{$f2bin[$i]});
    }
    $SIM = sprintf("%4.3f",($NAB/$d)) if($d > 0) ;
    return($SIM);
}

### Ligand Chemical similarity
sub tannimoto
{
        my($f1,$f2)=@_;
        my @f1bin=split(//,$f1);
        my @f2bin=split(//,$f2);
        my $bins = $#f1bin;
        $bins = $#f2bin if($#f2bin < $bins);
        my $NA=0; my $NB=0; my $NAB=0;  my $Tan=0;
        for(my $i=0;$i<=$bins;$i++)
        {
            $NA++  if($f1bin[$i] ne '0');
            $NB++  if($f2bin[$i] ne '0');
            $NAB++ if(($f1bin[$i] eq $f2bin[$i]) && ($f1bin[$i] ne '0'))
        }
        my $d = ($NA+$NB-$NAB);
        $Tan = sprintf("%4.3f",($NAB/$d)) if($d > 0) ;
        return($Tan);
}

### Mean
sub Mean 
{
        my($hash_ref,$arr_ref,$n)=@_;
        my %x = %$hash_ref;
        my @arr=@$arr_ref;
        my $sum=0;my $i=0;
        foreach my $a(@arr)
        {
            $sum += $x{$a};
        }

        my $m=$sum/($n);
        return $m;
}

### SD
sub SD 
{
    my($hash_ref,$arr_ref,$mean_ref,$n)=@_;
    my %x = %$hash_ref;
    my @arr=@$arr_ref;
    my $i=0; my $sum=0;
    foreach my $a(@arr)
    {
        $sum=$sum+($x{$a}-$$mean_ref)*($x{$a}-$$mean_ref);
    }
    my $Std=sqrt($sum);
    $Std=sqrt($sum/($n-1))if($n >1);
    return $Std;
}
                       
sub getCscore
{ 
    my($dir,$cscore_ref)=@_;
    my %NAME=(1=>"QQQ",2=>"IIIe",3=>"WWW",4=>"VVV",5=>"BBB",6=>"GGGd",7=>"IIIj",8=>"AAA",9=>"EEE",10=>"UUU",11=>"CCC",12=>"NNNd",13=>"OOO");
    
    my @inits=("QQQ","IIIe","WWW","VVV","BBB","GGGd","IIIj","AAA","EEE","UUU","CCC","NNNd","OOO");
						 
    my %zscore0=('QQQ'=>5.8,'IIIe'=>9.5,'WWW'=>6,'VVV'=>7.0,'BBB'=>3.2,'GGGd'=>9.3,'IIIj'=>11.5,'AAA'=>19,'EEE'=>7,'UUU'=>7.0,'CCC'=>15,'NNNd'=>7.0,'OOO'=>25,'III'=>40);	 	    
    my $init="init.dat";

    if(-s "$dir/init2.dat"){
        $init="init2.dat";
    }
    else{
        $init="init.dat";
    }


    my $M=13; my $k =0; my $z_sum =0; 
    for(my $i=1;$i<=$M;$i++){
	open(FH,"$dir/$init");
	<FH>=~/(\d+)/;
	my $nt=$1;
	my %zscore=();
	my $n=0;my %L=();
	for(my $j=1;$j<=$nt;$j++){
	    <FH>=~/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
	    $L{$j}=$1;
	    my $from=$5;
	    if($from eq "$NAME{$i}"){
		$n++;
		$zscore{$n}=$2;
                if($zscore{$n} < 0){$zscore{$n}=0;}
	    }
	    for(my $o=1;$o<=$L{$j};$o++){
		<FH>;
	    }
	    <FH>;
	}
      pos2:;
	close(FH);
	
	if($n > 0){
	    my @zscore_keys=sort{$zscore{$b}<=>$zscore{$a}} keys %zscore;
	    $k++;
	    my $z_max=$zscore{$zscore_keys[0]};
	    if($NAME{$i} eq "III"){
		$z_max=$zscore{1};
	    }
	    my $z_nor=$z_max/$zscore0{$NAME{$i}};
	    $z_sum+=$z_nor;
	}
    }
    my $zn=$z_sum/$k;
    my $rst="$dir/rst.dat";
    if(-s "$dir/rst.dat"){$rst="$dir/rst.dat";}
    elsif(-s "$dir/cluster/rst.dat"){$rst="$dir/cluster/rst.dat";}
    elsif(-s "$dir/cluster_new/rst.dat"){$rst="$dir/cluster_new/rst.dat";}
    elsif(-s "$dir/CLUSTER/rst.dat"){$rst="$dir/CLUSTER/rst.dat";}
    elsif(-s "$dir/cluster2/rst.dat"){$rst="$dir/cluster2/rst.dat";}
    else{print "Error: $rst was not found\n"; $$cscore_ref{1}=1;goto norst;}
    my $nc=0;my $nstr=0;
    open(FH,"<$rst"); 
    while(my $line=<FH>){
	my $Lch=$1  if($line=~/Modeling Length:\s+(\d+)/);
	if($line=~/Number of structure in use\=\s*(\S+)/){$nstr=$1;}
	if($line=~/Number of clusters\:\s+(\S+)/){
	    $nc=$1; $nc=5    if($nc>5);
	   
	}	
	my $Rcin='';
	if($line=~/C-----/){	   
	    for(my $i=1;$i<=$nc;$i++){
		my $line2=<FH>;
		$line2=~/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
		my $Nin=$2;  		
		my $cut=0.8;
		if($i==1){
		    $Rcin=$4;
		    $cut=0.8;
		    $Rcin=$cut if($Rcin<$cut);
		}	
		my $density=$Nin/$nstr/$Rcin;  
		$$cscore_ref{$i}=sprintf("%6.3f",log($zn*$density));               
	    }
	}
    }
    close(FH);
    norst:;
}

### Running time
sub runhour
{
    my($time1)=@_;
    my $time2=time();
    my $time=($time2-$time1)/3600;
    return($time);
}   
