#!/usr/bin/perl
use Math::Trig;


`hostname`=~/(\S+)/;
my $node=$1;
my $datetime=`date`;
my $path=`pwd`;
my $start=time();
print "Hostname: $node\n";
print "Started : $datetime\n";
print "Path    : $path\n";




#=pod
$data_dir       = "/nfs/amino-home/zhanglabs/IonCom/output/51"; 
$s              = "51"; 
my $model       = "protein.pdb"; 
my $seqname     = "seq.txt";
my $user        = "zhanglabs";
$run            = "benchmark";
my $bindir      = "/nfs/amino-home/zhanglabs/IonCom/bin/COACH/bin"; ##COACH bindir
#=cut

=pod
$data_dir       = "/home/zhanglabs/COACH/output/CH000001";
$s              = "CH000001";
$model       = "protein.pdb";
$seqname     = "seq.txt";
$user        = "zhanglabs";
$run            = "real";
$bindir      = "/home/zhanglabs/COACH/bin"; ##COACH bindir
=cut

my $home   = "/nfs/amino-home";
my $libdir = "/nfs/amino-library";


$ENV{FINDSITELIB}="$libdir/functionlib/FINDSITE";
$ENV{FINDSITEMAP}="$bindir/hhsearchdb_mapping.cls";

%ts=(
     'GLY'=>'G',
     'ALA'=>'A',
     'VAL'=>'V',
     'LEU'=>'L',
     'ILE'=>'I',
     'SER'=>'S',
     'THR'=>'T',
     'CYS'=>'C',
     'MET'=>'M',
     'PRO'=>'P',
     'ASP'=>'D',
     'ASN'=>'N',
     'GLU'=>'E',
     'GLN'=>'Q',
     'LYS'=>'K',
     'ARG'=>'R',
     'HIS'=>'H',
     'PHE'=>'F',
     'TYR'=>'Y',
     'TRP'=>'W',

     'ASX'=>'B',
     'GLX'=>'Z',
     'UNK'=>'X',

     'G'=>'GLY',
     'A'=>'ALA',
     'V'=>'VAL',
     'L'=>'LEU',
     'I'=>'ILE',
     'S'=>'SER',
     'T'=>'THR',
     'C'=>'CYS',
     'M'=>'MET',
     'P'=>'PRO',
     'D'=>'ASP',
     'N'=>'ASN',
     'E'=>'GLU',
     'Q'=>'GLN',
     'K'=>'LYS',
     'R'=>'ARG',
     'H'=>'HIS',
     'F'=>'PHE',
     'Y'=>'TYR',
     'W'=>'TRP',

     'a'=>'CYS',
     'b'=>'CYS',
     'c'=>'CYS',
     'd'=>'CYS',
     'e'=>'CYS',
     'f'=>'CYS',
     'g'=>'CYS',
     'h'=>'CYS',
     'i'=>'CYS',
     'j'=>'CYS',
     'k'=>'CYS',
     'l'=>'CYS',
     'm'=>'CYS',
     'n'=>'CYS',
     'o'=>'CYS',
     'p'=>'CYS',
     'q'=>'CYS',
     'r'=>'CYS',
     's'=>'CYS',
     't'=>'CYS',
     'u'=>'CYS',
     'v'=>'CYS',
     'w'=>'CYS',
     'x'=>'CYS',
     'y'=>'CYS',
     'z'=>'CYS',

     'B'=>'ASX',
     'Z'=>'GLX',
     'X'=>'CYS',
    );

################# directories #############################
$n_temp         = 40;
$idcut0         = 10.3;

if($run eq "benchmark")
{
    $idcut0=0.3;
}

################## findsite settings #########################
my $hhdblst="hhdb.lst";

################# directories #############################
$HHM_bin="$libdir/bin/HHM1.5";
$HHM_db="$libdir/HHM";

if(!-s "$data_dir/$model")
{
    die "$model is missed!\n";
}

################ working directory ########################
my $modelname   = $model; $modelname=~s/\.pdb//;
$work_dir       = "/tmp/$user/FINDSITE_$s\_$modelname";
my $outdir      = "$data_dir/$modelname/findsite";
`mkdir -p $outdir` if(!-d "$outdir");

`/bin/mkdir -p $work_dir`;
chdir "$work_dir";
`/bin/rm -f $work_dir/*`;
`/bin/cp $libdir/bin/align ./align`;
`/bin/cp $bindir/$hhdblst .`;
`/bin/cp $bindir/findsite .`;
`/bin/cp $data_dir/$model .`;

################ make fasta sequence file #################
if(!-s "$data_dir/$seqname")
{
    my $queryseq=&getfasta("$data_dir/$model");
    open(FSEQ, ">$data_dir/$seqname");
    print FSEQ ">seq\n";
    print FSEQ "$queryseq\n";
    close(FSEQ);
}
@seqtxts=`cat $data_dir/$seqname`;
$sequence="";
foreach $seqtxt(@seqtxts)
{
    if($seqtxt =~ /\>/) # there is a '>'
    {
	#do nothing
    }
    else
    {
	$seqtxt =~ s/\s//mg; #replace white-space character
	$seqtxt =~ s/\n//mg; #replace '\n' character
	$sequence = $sequence.$seqtxt; #add $seqtex to $sequence
    }
}
$Lch=length $sequence;

open(seq, ">protein.seq");
printf seq ">protein\n";
for($i=1; $i<=$Lch; $i++)
{
    $a=substr($sequence, $i-1, 1);
    printf seq "$a";
    $seqQ{$i}=$a;   #only for check
    if($i == int($i/60)*60)
    {
	printf seq "\n";
    }
}
printf seq "\n";
close(seq);

################# read old hhsearch db list ############
open(FDB, "$hhdblst") or die "$hhdblst is missed!\n";
my %hhdb=();
while(my $line = <FDB>)
{
    chomp($line);
    if($line =~ /\S+/)
    {
	$hhdb{$line}=1;
    }
}
close(FDB);

################ make protein.hhm #######################
$hhresult="$data_dir/protein\.out";
if(-s "$hhresult") #no pdb file for the template
{
    printf "running HHsearch (result exists, copy directly)....\n";
    `cp $hhresult .`;
}
else
{
    printf "running buildali.pl query.seq\n";
    
    `/nfs/amino-home/yzhang/library/yzhang/bin/HHM1.5/buildali_1.pl protein.seq -libdir $HHM_bin`;

    printf "running hhfilter .....\n";
    `$HHM_bin/hhfilter -i protein.a3m -o protein.f3m`;
    printf "running hhmake .....\n";
    `$HHM_bin/hhmake -i protein.f3m -o protein.hhm`; #make hhm file
    printf "Calibrate protein.hhm.....\n";
    `$HHM_bin/hhsearch -cal -loc -mapt 0 -i protein.hhm -d $HHM_db/cal.hhm`; #add EVD
    
    ########### scan protein.hhm through HHM database #######
    printf "running hhsearch .....\n";
    system("$HHM_bin/hhsearch -loc -mapt 0 -i protein.hhm -d $HHM_db/db.hhm -o protein.out -z 20000 -Z 30000 -b 20000 -B 30000 -v 0");
    `/bin/cp protein.out            $data_dir/protein.out`;	
}
################ calculate Z-score ######################
printf "computing Z-score .....\n";

open(dist,"protein.out");

$i=0;
$flag=0;
while($line = <dist>)
{
    if($line =~ /No Hit                             Prob E-value/) #the hhsearch result: list of template and scores
    {
	while($line = <dist>)
	{
	    if($line =~ /^No\s+\d+/)
	    {
		$flag=1;
		last;
	    }
	    elsif($line =~ /(\d+)\s+(\S+)/)
	    {
		my $NO1=$1;
		my $name=$2;
		
		if(!exists $hhdb{$name})
		{
		    #print "$name is a new entry, ignore\n";
		    next;
		}

		$i++;
		$NO{$i} = $NO1;
		$T_name{$i} = $name;
		$score{$i} = substr($line, 58, 6);
		$score_a += $score{$i};
		$score_a2 += $score{$i}**2;
	    }
	}
	
	if($flag==1)
	{
	    last;
	}
    }
}
close(dist);


$N_hit=$i;
$score_a/=$N_hit;
$score_a2/=$N_hit;
$dev=sqrt($score_a2-$score_a**2); #standard devitation
for($i=1; $i<=$N_hit; $i++)
{
    $zscore{$i}=-($score_a-$score{$i})/$dev;
    if($i>1)
    {
	if($zscore{$i} > $zscore{$i-1}) #why do this?
	{
	    $zscore{$i}=$zscore{$i-1}-0.01;
	}
    }
}

###########################################################
##### create template file 'init.dat' #####################
###########################################################
printf "creating template file init.dat .....\n";
open(init, ">init.dat");
open(tname, ">templates.lst");
$i_t=0;
for($i=1; $i<=$N_hit; $i++)
{
    $zscore_value = $zscore{$i};
    $template_name = $T_name{$i};
    $pdb = "$HHM_db/$template_name\.pdb";
    if(!-s "$pdb") #no pdb file for the template
    {
	next;
    }
    
    ############ check homology ##############
    if($idcut0 < 0.999)
    {
	$align_rst = `./align protein.seq $pdb 2`;
	if($align_rst =~ /Identical length\:\s+(\d+)/)
	{
	    $id = $1/$Lch;
	    if($id >= $idcut0)
	    {
		next;
	    }
	}
    }
    
    
    ###### read alignment ####################
    #do this for copying coordinates from template aligned residues and query aligned residues
    open(align, "protein.out"); #note that this file will be open-closed for $N_hit times
    $L_ali=0;
    $flag=0;
    while($line = <align>)
    {		
	if($line =~ /^No\s+$NO{$i}/) #the i-th template
	{
	    while($line = <align>)
	    {
		if($line =~ /No\s+\d+/) #a new protein, so end while
		{
		    $flag=1;
		    last;
		}
		if($line =~ /^Q protein\s+(\d+)\s+(\S+)\s+(\d+)\s+\(/) #aligned query sequence
		{
		    $nQ=$1-1; #the begining index of the query sequence
		    $sequenceQ=$2;
		}
		if($line =~ /T\s+$template_name\s+(\d+)\s+(\S+)/)
		{
		    $nT=$1-1; #the begining index of the template sequence
		    $sequenceT=$2;
		    ###
		    $L=length $sequenceT;
		    for($j=1; $j<=$L; $j++)
		    {
			$sQ=substr($sequenceQ, $j-1, 1);
			$sT=substr($sequenceT, $j-1, 1);
			$nQ++ if($sQ ne "-");
			$nT++ if($sT ne "-");
			if($sQ ne "-" && $sT ne "-") #record the aligned residues for query sequence and template sequence
			{
			    $L_ali++;
			    $resQ{$L_ali}=$nQ;
			    $resT{$L_ali}=$nT;
			    $seqQ{$L_ali}=$sQ;
			    $seqT{$L_ali}=$sT;
			}
		    }#for($j=1; $j<=$L; $j++);
		}#if($line =~ /T\s+$template_name\s+(\d+)\s+(\S+)/);
	    }#while($line = <align>);
	}#if($line=~/^No\s+$NO{$i}/);
	if($flag==1)
	{
	    last;
	}
    }#while($line = <align>);
    close(align);
    
    ############ get coordinates from PDB file ---------->
    open(pdb, "$pdb");
    $n=0;
    $L_ali1=0;
    while($line = <pdb>)
    {
	if(substr($line, 12, 4) =~ /CA/) #alpha carbon
	{
	    $n++;
	    substr($line, 22, 4) =~ /(\d+)/;
	    $tmp=$1;
	    ######## check whether the coordinate exist ---------->
	    for($j=1; $j<=$L_ali; $j++)
	    {
		if($tmp == $resT{$j} && $resQ{$j}<$Lch)
		{
		    $L_ali1++;
		    $resQ1{$L_ali1}=$resQ{$j};
		    $seqQ1{$L_ali1}=$seqQ{$j};
		    $resT1{$L_ali1}=$resT{$j};
		    $seqT1{$L_ali1}=$seqT{$j};
		    $xyz{$L_ali1}=substr($line, 30, 24);
		    ##### check template sequence ----->
		    $tmp1=substr($line, 17, 3);
		    if($tmp1 ne $ts{$seqT1{$L_ali1}})
		    {
			printf "Template mismatch warning: $template_name: $tmp1 ne $ts{$seqT1{$L_ali1}}!\n";
		    }
		    ##### check query sequence ----->
		    $tmp1=substr($sequence, $resQ1{$L_ali1}-1, 1);
		    if($tmp1 ne $seqQ1{$L_ali1})
		    {
			printf "Query mismatch warning: $template_name: $tmp1 ne $seqQ1{$L_ali1}!\n";
		    }
		    last;
		}#if($tmp == $resT{$j} && $resQ{$j}<$Lch);
	    }#for($j=1; $j<=$L_ali; $j++);
	}#if(substr($line, 12, 4) =~ /CA/);
    }
    close(pdb);
    if($L_ali1>=5)
    {
	############## output init.dat ------------------>
	$i_t++;
	printf init "%5d %8.3f %5d   %6s\n", $L_ali1, $zscore_value, $i_t, $template_name;
	for($j=1; $j<=$L_ali1; $j++)
	{
	    printf init "ATOM  %5s  CA  %3s  %4d    %24s%5d %3s\n",
	    $j+$i_t*1000, $ts{$seqQ1{$j}}, $resQ1{$j}, $xyz{$j}, $resT1{$j}, $ts{$seqT1{$j}};
	}
	printf init "TER\n";
	
	#save template name for running findsite
	printf tname "$template_name\n";
	
	if($i_t >= $n_temp) #maximum number of templates arrived
	{
	    last;
	}
    }
}#for($i=1; $i<=$N_hit; $i++);
close(init);
close(tname);
`/bin/cp templates.lst          $outdir/templates_findsite.lst`;

open(init1,">init1.dat");
printf init1 "%5d %5d (N_temp, Lch)\n", $i_t, $Lch;
close(init1);
`cat init.dat >>init1.dat`;

`/bin/cp init1.dat            $outdir/init.IIIe`;
#`/bin/cp -r $work_dir/            $data_dir/threading/`;



#################### run findsite ##########################

printf "running findsite ........\n";
system("./findsite -s $model -t templates.lst -b $idcut0 -o $modelname");
my $findir = ".";
&fin_centers($findir, $model, "$modelname.pockets.dat", "$modelname.ligands.sdf", "templates.inf");
`/bin/cp templates.inf    $outdir/.`;
`/bin/cp lig_*.pdb $outdir/`;

`/bin/cp -r *.pockets.dat    $outdir/.`;
`/bin/cp -r *.pockets.pdb    $outdir/.`;
`/bin/cp -r *.alignments.dat $outdir/.`;
`/bin/cp -r *.ligands.sdf    $outdir/.`;

################# endding procedure ######################
my $time=`date`;
printf "ending time: $time";
`sync`;
sleep(5);
`rm -fr $work_dir`;

exit();



sub getfasta
{
    my $seq="";
    my ($pdb)=@_;


    my %AA=(
	    'ALA'=>'A', 'ARG'=>'R', 'ASN'=>'N', 'ASP'=>'D', 'CYS'=>'C', 'GLN'=>'Q', 'GLU'=>'E', 'GLY'=>'G',
	    'HIS'=>'H', 'ILE'=>'I', 'LEU'=>'L', 'LYS'=>'K', 'MET'=>'M', 'PHE'=>'F', 'PRO'=>'P', 'SER'=>'S',
	    'THR'=>'T', 'TRP'=>'W', 'TYR'=>'Y', 'VAL'=>'V', 'ASX'=>'N', 'GLX'=>'Q', 'UNK'=>'G',
	    'A'=>'ALA', 'R'=>'ARG', 'N'=>'ASN', 'D'=>'ASP', 'C'=>'CYS', 'Q'=>'GLN', 'E'=>'GLU', 'G'=>'GLY',
	    'H'=>'HIS', 'I'=>'ILE', 'L'=>'LEU', 'K'=>'LYS', 'M'=>'MET', 'F'=>'PHE', 'P'=>'PRO', 'S'=>'SER',
	    'T'=>'THR', 'W'=>'TRP', 'Y'=>'TYR', 'V'=>'VAL', 'B'=>'ASX', 'Z'=>'GLX', 'X'=>'UNK');

    open(FPDB, $pdb) or die "$pdb is missed!\n";
    my %resatoms=();
    while(my $line = <FPDB>)
    {
        if($line =~ /^ATOM/i)
        {
            my $res_name  = substr($line, 17, 3);
            my $res_no    = substr($line, 22, 5);
            my $check     = $res_no .  substr($line, 12, 4); #atom

            if(!exists $resatoms{$check}) ##check for alternative position atoms
            {
                $resatoms{$check}=1;

                my $atom_name=substr($line, 12, 4);
                if($atom_name eq ' CA ')
                {
                    $seq = $seq. $AA{$res_name};
                }
            }
        }
    }
    close(FPDB);

    return $seq;
}





sub fin_centers
{
    my ($dir, $pdbfile, $resultfile, $ligfile, $outname)=@_;
    

    if(!-s "$dir/$pdbfile")
    {
	print "pdb file is missed\n";
	return;
    }

    if(!-s "$dir/$resultfile")
    {
	print "result file is missed\n";
	return;
    }

    if(!-s "$dir/$ligfile")
    {
	print "ligand sdf file is missed\n";
	return;
    }


    my @pdbbuff=();
    open(PDB, "$dir/$pdbfile");
    while(my $line = <PDB>)
    {
	if($line =~ /^ATOM/)
	{
	    chomp($line);
	    push(@pdbbuff, $line);
	}
    }
    close(PDB);

    my %queryCA = &load_Ca($dir, $pdbfile);



    my %tmscore=();
    open(PDB, "$dir/$resultfile");
    while(my $line = <PDB>)
    {
	if($line =~ /^TEMPLATE\s+\d+\s+(\S+)\s+\d+\s+\d+\s+\S+\s+\S+\s+(\S+)/)
	{
	    $tmscore{$1}=$2;	    
	}
    }
    close(PDB);

    my $count=0;
    my %lignum=();
    open(PDB, "$dir/$ligfile");
    while(my $line = <PDB>)
    {
	if($line =~ /^>  <MOLID>/)
	{
	    $line = <PDB>;
	    chomp($line);
	    $count++;
	    $lignum{$count}=$line;	    
	}
    }
    close(PDB);



    open(OUT, ">$outname");

    `babel $dir/$ligfile findsite_lig.pdb`;
    my @ligall=`cat findsite_lig.pdb`;

    $count=0;
    my @lig1=();
    foreach my $line(@ligall)
    {
	chomp($line);
	if($line=~/^HETATM/ || $line=~/^ATOM/)
	{
	    substr($line, 0, 6)="HETATM";
	    push(@lig1, $line);
	}

	if($line =~ /^ENDMDL/)
	{   
	    my @res = &ProtLigContact(\@pdbbuff, \@lig1);	    
	    $count++;
	    

	    my $x=0; my $y=0; my $z=0; 
	    foreach my $r(@res)
	    {
		$x += $queryCA{$r, 1};
		$y += $queryCA{$r, 2};
		$z += $queryCA{$r, 3};
	    }
	    if(@res>2)
	    {
		$x = $x/@res;
		$y = $y/@res;
		$z = $z/@res;
		my $PBS=join(",", @res);

		my $pdb=lc(substr($lignum{$count}, 0, 4)) . substr($lignum{$count}, 4, 1);
		my $pdb1=substr($lignum{$count}, 0, 5);
		printf OUT "FIN\t$pdb\t$lignum{$count}\t%.3f %.3f %.3f\t%.2f :$PBS\n", $x, $y, $z, $tmscore{$pdb1};
		
		open(FTMP, ">lig_$lignum{$count}.pdb");
		print FTMP join("\n", @lig1);
		print FTMP "\nTER FINDSITE $lignum{$count}\n";
		close(FTMP);
	    }
	    @lig1=();
	}        
    }    
    close(OUT);
    `rm -f findsite_lig.pdb`;
}


sub dist
{
    my ($x1, $y1, $z1, $x2, $y2, $z2)=@_;
    my $dist =sqrt(($x1 - $x2)**2 + ($y1 - $y2)**2 + ($z1 - $z2)**2);
    return($dist);
}


### Input: a receptor structure + a ligand structure
### Output: the residue numbers in contact with the ligand
sub ProtLigContact
{
    my ($prot_ref, $lig_ref)=@_;
    my @protein  =@$prot_ref;
    my @ligand   =@$lig_ref;

    my %vdw=(
	    'AC'=>'2.00','Ac'=>'2.00','AG'=>'1.72','Ag'=>'1.72','AL'=>'2.00','Al'=>'2.00','AM'=>'2.00','Am'=>'2.00',
	    'AR'=>'1.88','Ar'=>'1.88','AS'=>'1.85','As'=>'1.85','AT'=>'2.00','At'=>'2.00','AU'=>'1.66','Au'=>'1.66',
	    'B' =>'2.00','BA'=>'2.00','Ba'=>'2.00','BE'=>'2.00','Be'=>'2.00','BH'=>'2.00','Bh'=>'2.00','BI'=>'2.00',
	    'Bi'=>'2.00','BK'=>'2.00','Bk'=>'2.00','Br'=>'1.85','BR'=>'1.85','C' =>'1.70','CA'=>'2.00','Ca'=>'2.00',        
	    'CD'=>'1.58','Cd'=>'1.58','CE'=>'2.00','Ce'=>'2.00','CF'=>'2.00','Cf'=>'2.00','CL'=>'1.75','Cl'=>'1.75',      
	    'CM'=>'2.00','Cm'=>'2.00','CO'=>'2.00','Co'=>'2.00','CR'=>'2.00','Cr'=>'2.00','CS'=>'2.00','Cs'=>'2.00',      
	    'CU'=>'1.40','Cu'=>'1.40','DB'=>'2.00','Db'=>'2.00','DS'=>'2.00','Ds'=>'2.00','DY'=>'2.00','Dy'=>'2.00',      
	    'ER'=>'2.00','Er'=>'2.00','ES'=>'2.00','Es'=>'2.00','EU'=>'2.00','Eu'=>'2.00','F' =>'1.47','Fe'=>'2.00',
	    'FE'=>'2.00','FM'=>'2.00','Fm'=>'2.00','Fr'=>'2.00','FR'=>'2.00','GA'=>'1.87','Ga'=>'1.87','H' =>'1.09',      
	    'GD'=>'2.00','Gd'=>'2.00','GE'=>'2.00','Ge'=>'2.00','HE'=>'1.40','He'=>'1.40','HF'=>'2.00','Hf'=>'2.00',      
	    'HG'=>'1.55','Hg'=>'1.55','HO'=>'2.00','Ho'=>'2.00','HS'=>'2.00','Hs'=>'2.00','I' =>'1.98','K' =>'2.75',      
	    'IN'=>'1.93','In'=>'1.93','IR'=>'2.00','Ir'=>'2.00','KR'=>'2.02','Kr'=>'2.02','LA'=>'2.00','La'=>'2.00',    
	    'LI'=>'1.82','Li'=>'1.82','LR'=>'1.50','Lr'=>'1.50','LU'=>'2.00','Lu'=>'2.00','MD'=>'2.00','Md'=>'2.00',      
	    'MG'=>'1.73','Mg'=>'1.73','MN'=>'2.00','Mn'=>'2.00','MO'=>'2.00','Mo'=>'2.00','MT'=>'2.00','Mt'=>'2.00',      
	    'N' =>'1.55','O' =>'1.52','NA'=>'2.27','Na'=>'2.27','NB'=>'2.00','Nb'=>'2.00','ND'=>'2.00','Nd'=>'2.00',      
	    'NE'=>'1.54','Ne'=>'1.54','NI'=>'1.63','Ni'=>'1.63','NO'=>'2.00','No'=>'2.00','NP'=>'2.00','Np'=>'2.00',
	    
	    'OS'=>'2.00','Os'=>'2.00','P' =>'1.80','S' =>'1.80','PA'=>'2.00','Pa'=>'2.00','PB'=>'2.02','Pb'=>'2.02',      
	    'PD'=>'1.63','Pd'=>'1.63','PM'=>'2.00','Pm'=>'2.00','PO'=>'2.00','Po'=>'2.00','PR'=>'2.00','Pr'=>'2.00',   
	    'PT'=>'1.72','Pt'=>'1.72','PU'=>'2.00','Pu'=>'2.00','RA'=>'2.00','Ra'=>'2.00','RB'=>'2.00','Rb'=>'2.00',      
	    'RE'=>'2.00','Re'=>'2.00','RF'=>'2.00','Rf'=>'2.00','RH'=>'2.00','Rh'=>'2.00','RN'=>'2.00','Rn'=>'2.00',      
	    'RU'=>'2.00','Ru'=>'2.00','SB'=>'2.00','Sb'=>'2.00','SC'=>'2.00','Sc'=>'2.00','SE'=>'1.90','Se'=>'1.90',     
	    'SG'=>'2.00','Sg'=>'2.00','SI'=>'2.10','Si'=>'2.10','SM'=>'2.00','Sm'=>'2.00','SN'=>'2.17','Sn'=>'2.17',      
	    'SR'=>'2.00','Sr'=>'2.00','TA'=>'2.00','Ta'=>'2.00','TB'=>'2.00','Tb'=>'2.00','TC'=>'2.00','Tc'=>'2.00',      
	    'TE'=>'2.06','Te'=>'2.06','TH'=>'2.00','Th'=>'2.00','TI'=>'2.00','Ti'=>'2.00','TL'=>'1.96','Tl'=>'1.96',      
	    'TM'=>'2.00','U' =>'1.86','V' =>'2.00','W' =>'2.00','XE'=>'2.16','Xe'=>'2.16','YB'=>'2.00','Yb'=>'2.00',           
	    'ZN'=>'1.39','Zn'=>'1.39','ZR'=>'2.00','Zr'=>'2.00','Y' =>'2.00','D' =>'1.00','XD'=>'1.09','O'=>'1.52');  


    my @contacts =();
    my @in=(); my %seen=(); my @out=(); my %NC=();
    foreach my $line1(@protein)
    {
	
	my $vdw1=1.75;
	my $res_no= substr($line1, 22, 5); $res_no =~ s/\s+//;
	my $atom1 = substr($line1, 12, 2); $atom1 =~ s/[\d+|\s+]//mg;  ##atom name
	if($atom1 =~/H/g) #ignore hydrogen
	{
	    next;
	}


	my $x1=substr($line1, 30, 8);
	my $y1=substr($line1, 38, 8);
	my $z1=substr($line1, 46, 8);
	if(exists $vdw{$atom1})
	{
	    $vdw1= $vdw{$atom1};
	}
	foreach my $line2(@ligand)
	{
	    my $vdw2=2.00;
	    my $atom2 = substr($line2, 12, 2); $atom2 =~ s/[\d+|\s+]//mg;
	    my $x2=substr($line2, 30, 8);
	    my $y2=substr($line2, 38, 8);
	    my $z2=substr($line2, 46, 8);
	    if(exists $vdw{$atom2})
	    {
		$vdw2= $vdw{$atom2};
	    }
	    
	    my $distcut     = $vdw1 + $vdw2 + 0.5;
	    my $distance    = dist($x1, $y1, $z1, $x2, $y2, $z2);
	    if(($distance <= $distcut))
	    {
		$NC{$res_no}++;
		push(@in, sprintf("%d", $res_no));
		last;
	    }
	}
    }   
    @out = grep(!$seen{$_}++, @in);  ###remove repeat element
    @contacts=sort { $a <=> $b } @out;

    return @contacts;
}


sub load_Ca
{    
    my($dir, $pdbfile) = @_;

    if(!-s "$dir/$pdbfile")
    {
	print "PDB file is missed\n";
	exit;
    }
    
    open(QUE, "$dir/$pdbfile");
    my %xyz=();
    my $i=0;
    my %resatoms=();
    
    while(my $line = <QUE>)
    {
	if($line =~ /^ATOM/)
	{
	    my $atom_name=substr($line, 12, 4);
	    if($atom_name eq ' CA ')
	    {		    
		my $res_no = substr($line, 22, 5); $res_no =~ s/\s+//g;
		my $check  = $res_no .  $atom_name;
		if(!exists $resatoms{$check}) #ignore alternative location of atoms
		{
		    $resatoms{$check}=1;

		    $xyz{$res_no, 1} = substr($line, 30, 8);
		    $xyz{$res_no, 2} = substr($line, 38, 8);
		    $xyz{$res_no, 3} = substr($line, 46, 8);
		}
	    }
	}
    }
    close(QUE);

    return %xyz;
}
